@@ -1,5 +1,5 @@
 ;;; emacspeak-eterm.el --- Speech enable eterm -- Emacs' terminal emulator  term.el
-;;; $Id: emacspeak-eterm.el,v 16.0 2002/05/03 23:31:23 raman Exp $
+;;; $Id: emacspeak-eterm.el,v 18.0 2003/04/29 21:17:06 raman Exp $
 ;;; $Author: raman $ 
 ;;; Description:  Emacspeak extension to speech enable eterm. 
 ;;; Keywords: Emacspeak, Eterm, Terminal emulation, Spoken Output
@@ -8,14 +8,14 @@
 ;;; LCD Archive Entry:
 ;;; emacspeak| T. V. Raman |raman@cs.cornell.edu 
 ;;; A speech interface to Emacs |
-;;; $Date: 2002/05/03 23:31:23 $ |
-;;;  $Revision: 16.0 $ | 
+;;; $Date: 2003/04/29 21:17:06 $ |
+;;;  $Revision: 18.0 $ | 
 ;;; Location undetermined
 ;;;
 
 ;;}}}
 ;;{{{  Copyright:
-;;;Copyright (C) 1995 -- 2002, T. V. Raman 
+;;;Copyright (C) 1995 -- 2003, T. V. Raman 
 ;;; Copyright (c) 1994, 1995 by Digital Equipment Corporation.
 ;;; All Rights Reserved. 
 ;;;
@@ -39,7 +39,6 @@
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;{{{ required packages:
 
-
 ;;; Commentary:
 ;;{{{  Introduction:
 ;;; This module makes eterm talk.
@@ -53,26 +52,11 @@
 ;; 
 ;;; Code:
 
-(eval-when-compile (require 'cl))
-(declaim  (optimize  (safety 0) (speed 3)))
-(require 'advice)
-(require 'custom)
-(eval-when-compile (require 'dtk-speak)
-                   (require 'dtk-voices )
-                   (require 'voice-lock)
-                   (require 'emacspeak-speak)
-                   (require 'emacspeak-sounds)
-                   (require 'emacspeak-keymap)
-                   (condition-case nil
-                       (progn
-                         (require 'term)
-                                        ;(require 'tshell)
-                         )
-                     (error nil )))
-
+(require 'emacspeak-preamble)
+(require 'term)
 ;;}}}
 ;;{{{ custom
-
+;;;###autoload
 (defgroup emacspeak-eterm nil
   "Terminal emulator for the Emacspeak Desktop."
   :group 'emacspeak
@@ -142,7 +126,7 @@
   (define-key emacspeak-eterm-keymap "\C-y" 'emacspeak-eterm-yank-window)
   (define-key emacspeak-eterm-keymap "\C-f"
     'emacspeak-eterm-set-focus-window)
-(define-key emacspeak-eterm-keymap "\C-a" 'emacspeak-eterm-toggle-focus-window)
+  (define-key emacspeak-eterm-keymap "\C-a" 'emacspeak-eterm-toggle-focus-window)
   (define-key emacspeak-eterm-keymap "\C-d" 'emacspeak-eterm-describe-window)
   (define-key emacspeak-eterm-keymap "\C-m" 'emacspeak-eterm-speak-window)
   (define-key emacspeak-eterm-keymap "r" 'emacspeak-eterm-toggle-review)
@@ -163,7 +147,8 @@ Useful when eterm is in review mode.")
 
 (defun emacspeak-eterm-setup-raw-keys ()
   "Setup emacspeak keys for raw terminal mode."
-  (declare (special term-raw-map 
+  (declare (special term-raw-map
+                    emacspeak-prefix term-raw-escape-map
                     emacspeak-eterm-keymap
                     emacspeak-eterm-raw-prefix))
   (when term-raw-map 
@@ -178,18 +163,22 @@ Useful when eterm is in review mode.")
       'emacspeak-eterm-maybe-send-raw)
     (define-key term-raw-map
       (concat emacspeak-eterm-prefix emacspeak-eterm-prefix) 'emacspeak-eterm-maybe-send-raw)
-    (define-key emacspeak-eterm-keymap emacspeak-eterm-raw-prefix term-raw-map)
+    (define-key emacspeak-eterm-keymap emacspeak-eterm-raw-prefix
+      term-raw-map)
+    ;;; handle emacs 21 wierdness 
+    (local-unset-key "\eO")
+    (local-unset-key "\e[")
     ))
 
 ;;}}}
 ;;{{{  voice definitions  for eterm  highlight, underline etc
 
-(defcustom emacspeak-eterm-highlight-personality 'harry
+(defcustom emacspeak-eterm-highlight-personality voice-bolden
   "Personality to show terminal highlighting."
   :type 'symbol
   :group 'emacspeak-eterm)
 
-(defcustom emacspeak-eterm-bold-personality 'paul-bold
+(defcustom emacspeak-eterm-bold-personality voice-bolden
   "Persnality to indicate terminal bold."
   :type 'symbol
   :group 'emacspeak-eterm)
@@ -204,7 +193,6 @@ Useful when eterm is in review mode.")
   :type 'symbol
   :group 'emacspeak-eterm)
 
-
 ;;}}}
 ;;{{{  functions
 
@@ -248,7 +236,6 @@ in a non eterm buffer if executed via C-e C-e"
      "Pointer at row %s column %s "
      (cdr coordinates ) (car coordinates ))))
 
-
 (defun emacspeak-eterm-speak-screen (&optional flag )
   "Speak the screen.  Default is to speak from the emacspeak pointer  to point.
 Optional prefix arg FLAG causes region above
@@ -751,7 +738,7 @@ Argument ETERM-WINDOW specifies a predefined eterm window."
               end (emacspeak-eterm-coordinates-to-position bottom-right))
         (setq contents
               (cond
-               ((and left-stretch right-stretch );; stretchable window
+               ((and left-stretch right-stretch ) ;; stretchable window
                 (goto-char start )
                 (beginning-of-line )
                 (setq start (point))
@@ -803,7 +790,7 @@ Argument ETERM-WINDOW specifies a predefined eterm window."
   "Vector of window positions.
 A terminal window is recorded by the  positions of its top left
 and bottom right.")
-
+;;;###autoload
 (defun emacspeak-eterm-record-window  (window-id top-left bottom-right
                                                  &optional right-stretch left-stretch )
   "Insert this window definition into the table of terminal windows.
@@ -884,10 +871,6 @@ Argument ID specifies the window."
        (emacspeak-eterm-return-window-contents
         (emacspeak-eterm-get-window id ))))))
 
-
-
-
-
 (defun emacspeak-eterm-yank-window (id)
   "Yank contents of  an eterm window at point."
   (interactive "nYank contents of window")
@@ -957,7 +940,6 @@ non-negative integer ")
         (setq emacspeak-eterm-focus-window window-id )
         (message "Set emacspeak eterm focus window  to %d " window-id )))))))
 
-
 (defun emacspeak-eterm-toggle-focus-window ()
   "Toggle active state of focus window."
   (interactive)
@@ -967,8 +949,8 @@ non-negative integer ")
     (setq emacspeak-eterm-focus-window 1))
   (dtk-stop)
   (emacspeak-auditory-icon
-     (if emacspeak-eterm-focus-window
-         'on 'off)))
+   (if emacspeak-eterm-focus-window
+       'on 'off)))
 
 (defun emacspeak-eterm-speak-predefined-window ()
   "Speak a predefined eterm window between 1 and 10."
@@ -1059,14 +1041,12 @@ all output to the terminal is automatically spoken.
 (defvar eterm-char-mode t
   "Flag indicating if eterm is in char mode.")
 
-
 (defvar emacspeak-eterm-pointer-mode t 
   "If T then the emacspeak pointer will not track the terminal cursor.
 Do not set this by hand.
 Use command emacspeak-eterm-toggle-pointer-mode bound to
 \\[emacspeak-eterm-toggle-pointer-mode].")
 
-
 (defadvice  term-emulate-terminal (around emacspeak pre act compile )
   "Record position, emulate, then speak what happened.
 Also keep track of terminal highlighting etc.
@@ -1096,12 +1076,7 @@ See command emacspeak-toggle-eterm-autospeak bound to
         (new-column nil )
         (old-point (point))
         (dtk-stop-immediately (not eterm-line-mode)))
-    (save-match-data 
-      (ad-enable-advice  'put-text-property 'after 'eterm )
-      (ad-activate 'put-text-property) 
-      ad-do-it
-      (ad-disable-advice  'put-text-property 'after 'eterm )
-      (ad-deactivate 'put-text-property))
+    ad-do-it
     (setq new-row (term-current-row )
           new-column (term-current-column )
           new-end (point-max))
@@ -1181,26 +1156,10 @@ there is terminal activity."
     ad-return-value )
   )
 
-(defadvice put-text-property (after eterm pre act) 
-  "Used by emacspeak to highlight successfully in eterm. "
-  (let ((start (ad-get-arg 0))
-        (end (ad-get-arg 1 ))
-        (prop (ad-get-arg 2))
-        (value (ad-get-arg 3 )))
-    (if (eq prop 'face) 
-        (save-match-data
-          (cond
-           ((eq value 'bold)
-            (put-text-property start end
-                               'personality emacspeak-eterm-bold-personality))
-           ((eq value 'highlight)
-            (put-text-property start end
-                               'personality emacspeak-eterm-highlight-personality ))
-           ((eq value 'term-underline-face )
-            (put-text-property start end
-                               'personality emacspeak-eterm-underline-personality))
-           (t  nil )))))
-  )
+(def-voice-font emacspeak-eterm-underline-personality
+  voice-lock-underline-personality
+  'term-underline
+  "Underline personality for eterm.")
 
 (defadvice term-line-mode (after emacspeak pre act)
   "Announce that you entered line mode. "
@@ -1242,7 +1201,6 @@ there is terminal activity."
   (when (interactive-p)
     (emacspeak-speak-line )))
 
-
 (defadvice term-previous-input (after emacspeak pre act)
   "Speak the line. "
   (when (interactive-p)
@@ -1347,7 +1305,7 @@ emacspeak-eterm-remote-hostnames")
 
 (eval-when (load)
   (emacspeak-eterm-load-remote-hosts-cache))
-
+;;;###autoload
 (defun emacspeak-eterm-cache-remote-host (host)
   "Add this hostname to cache of remote hostnames"
   (declare (special emacspeak-eterm-remote-hosts-table
@@ -1360,7 +1318,7 @@ emacspeak-eterm-remote-hostnames")
       (insert (format "%s\n" host))
       (save-buffer))
     (intern host emacspeak-eterm-remote-hosts-table)))
-
+;;;###autoload
 (defun emacspeak-eterm-remote-term (host )
   "Start a terminal-emulator in a new buffer."
   (interactive
