@@ -1,5 +1,5 @@
 ;;; emacspeak-table-ui.el --- Emacspeak's current notion of an ideal table UI
-;;; $Id: emacspeak-table-ui.el,v 16.0 2002/05/03 23:31:23 raman Exp $
+;;; $Id: emacspeak-table-ui.el,v 18.0 2003/04/29 21:18:07 raman Exp $
 ;;; $Author: raman $ 
 ;;; Description: Emacspeak table handling module
 ;;; Keywords:emacspeak, audio interface to emacs tables are structured
@@ -8,14 +8,14 @@
 ;;; LCD Archive Entry:
 ;;; emacspeak| T. V. Raman |raman@cs.cornell.edu
 ;;; A speech interface to Emacs |
-;;; $Date: 2002/05/03 23:31:23 $ |
-;;;  $Revision: 16.0 $ | 
+;;; $Date: 2003/04/29 21:18:07 $ |
+;;;  $Revision: 18.0 $ | 
 ;;; Location undetermined
 ;;;
 
 ;;}}}
 ;;{{{  Copyright:
-;;;Copyright (C) 1995 -- 2002, T. V. Raman 
+;;;Copyright (C) 1995 -- 2003, T. V. Raman 
 ;;; Copyright (c) 1995 by T. V. Raman  
 ;;; All Rights Reserved. 
 ;;;
@@ -37,19 +37,15 @@
 
 ;;}}}
 
-(eval-when-compile (require 'cl))
-(declaim  (optimize  (safety 0) (speed 3)))
-(require 'custom)
-(require 'emacspeak-table)
-(require 'emacspeak-tabulate)
-(eval-when-compile (require 'dtk-speak)
-(require 'emacspeak-speak)
-(require 'emacspeak-sounds))
-(require 'thingatpt)
 ;;{{{  Introduction
 
 ;;; User interface to tables
 
+;;}}}
+;;{{{ requires
+(require 'emacspeak-preamble)
+(require 'emacspeak-table)
+
 ;;}}}
 ;;{{{ define personalities
 
@@ -58,16 +54,15 @@
   :group 'emacspeak
   :prefix "emacspeak-table-")
 
-
-(defcustom emacspeak-table-column-header-personality 'paul-smooth
+(defcustom emacspeak-table-column-header-personality voice-smoothen
   "personality for speaking column headers."
   :type 'symbol
   :group 'emacspeak-table)
 
-(defcustom emacspeak-table-row-header-personality 'harry
+(defcustom emacspeak-table-row-header-personality voice-bolden
   "Personality for speaking row headers"
-:type 'symbol
-:group 'emacspeak-table)
+  :type 'symbol
+  :group 'emacspeak-table)
 
 ;;}}}
 ;;{{{  emacspeak table mode
@@ -220,7 +215,6 @@ specifies the filter"
                     (+ (/ width  2)
                        (window-hscroll))))))
 
-
 (defsubst  emacspeak-table-speak-coordinates ()
   "Speak current table coordinates."
   (interactive)
@@ -349,6 +343,8 @@ specifies the filter"
 Optional prefix arg prompts for a new filter."
   (interactive "P")
   (declare (special emacspeak-table-speak-row-filter
+                    voice-animate
+                    voice-smoothen
                     emacspeak-table))
   (unless (and  emacspeak-table-speak-row-filter
                 (listp emacspeak-table-speak-row-filter)
@@ -376,7 +372,7 @@ Optional prefix arg prompts for a new filter."
                     (emacspeak-table-current-row emacspeak-table)
                     token))
              (put-text-property 0 (length value)
-                                'personality 'paul-smooth value)
+                                'personality voice-smoothen  value)
              value)
             ((and (listp token)
                   (numberp (first token))
@@ -386,7 +382,7 @@ Optional prefix arg prompts for a new filter."
                     (first token)
                     (second token)))
              (put-text-property 0 (length value)
-                                'personality 'paul-smooth value)
+                                'personality voice-smoothen value)
              value)
             (t  (format "%s" token))))))
       emacspeak-table-speak-row-filter
@@ -443,7 +439,7 @@ Optional prefix arg prompts for a new filter."
 
 ;;}}}
 ;;{{{  opening a file of table data
-
+;;;###autoload 
 (defsubst emacspeak-table-prepare-table-buffer (table buffer
                                                       &optional filename)
   "Prepare tabular data."
@@ -491,7 +487,7 @@ Optional prefix arg prompts for a new filter."
   (switch-to-buffer buffer)
   (setq truncate-lines t)
   (message "Use Emacspeak Table UI to browse this table."))
-
+;;;###autoload
 (defun emacspeak-table-find-file (filename)
   "Open a file containing table data and display it in table mode.
 emacspeak table mode is designed to let you browse tabular data using
@@ -512,6 +508,7 @@ the documentation on the table browser."
     (kill-buffer data )
     (emacspeak-table-prepare-table-buffer table buffer filename )))
 
+;;;###autoload
 (defun emacspeak-table-find-csv-file (filename)
   "Process a csv (comma separated values) file. 
 The processed  data and presented using emacspeak table navigation. "
@@ -546,7 +543,7 @@ The processed  data and presented using emacspeak table navigation. "
     (kill-buffer scratch)
     (emacspeak-table-prepare-table-buffer table buffer
                                           filename )))    
-
+;;;###autoload
 (defun emacspeak-table-view-csv-buffer (&optional buffer-name)
   "Process a csv (comma separated values) data. 
 The processed  data and presented using emacspeak table
@@ -588,6 +585,7 @@ navigation. "
 
 ;;}}}
 ;;{{{ Processing a region of tabular data
+;;;###autoload
 (defun emacspeak-table-display-table-in-region (start end)
   "Recognize tabular data in current region and display it in table
 browsing mode in a a separate buffer.
@@ -600,7 +598,8 @@ CalTrain schedules.  Execute command `describe-mode' bound to
 the documentation on the table browser."
   (interactive "r")
   (declare (special positions))
-  (let ((buffer (get-buffer-create
+  (let ((workspace (get-buffer-create " table workspace  "))
+        (buffer (get-buffer-create
                  (format  "*table-%s*"
                           (or (buffer-name)
                               "scratch"))))
@@ -610,9 +609,19 @@ the documentation on the table browser."
         (j 0)
         (count 0)
         (row-start 1)
-        (column-start 1))
-    (setq table (emacspeak-table-make-table
-                 (ems-tabulate-parse-region start end)))
+        (column-start 1)
+        (text (buffer-substring start end)))
+    (save-excursion
+      (set-buffer workspace)
+      (erase-buffer)
+      (insert text)
+      (goto-char (point-min))
+      (flush-lines "^ *$")
+      (setq table (emacspeak-table-make-table
+		   (ems-tabulate-parse-region
+		    (point-min)
+		    (point-max)))))
+    (kill-buffer workspace)
     (save-excursion
       (set-buffer buffer)
       (let ((inhibit-read-only t))
@@ -916,8 +925,8 @@ table markup.")
 
 (defsubst emacspeak-table-markup-get-table (mode )
   (declare (special emacspeak-table-markup-table))
-  (or (cl-gethash mode emacspeak-table-markup-table)
-      (cl-gethash 'fundamental-mode emacspeak-table-markup-table)))
+  (or (gethash mode emacspeak-table-markup-table)
+      (gethash 'fundamental-mode emacspeak-table-markup-table)))
 
 ;;}}}
 ;;{{{  define table markup for the various modes of interest
@@ -941,7 +950,6 @@ table markup.")
                                    :col-end "</TD>\n"
                                    :col-separator ""))
 
-
 (emacspeak-table-markup-set-table 'latex2e-mode
                                   (emacspeak-table-make-markup
                                    :table-start "\\begin{tabular}\n"
@@ -986,11 +994,9 @@ table markup.")
                                    :col-end ""
                                    :col-separator "\t"))
 
-
-
 ;;}}}
 ;;{{{ copy and paste tables 
-
+;;;###autoload
 (defun emacspeak-table-copy-to-clipboard ()
   "Copy table in current buffer to the table clipboard.
 Current buffer must be in emacspeak-table mode."
@@ -1007,7 +1013,6 @@ emacspeak-table-mode. "))
     (message "Copied current table to emacspeak table clipboard."))
    (t (error "Cannot find table in current buffer"))))
 
-
 (defun emacspeak-table-paste-from-clipboard ()
   "Paste the emacspeak table clipboard into the current buffer.
 Use the major  mode of this buffer to  decide what kind of table
@@ -1131,7 +1136,6 @@ future  use."
     (error (message "Error loading resources from %s "
                     file))))
 
-
 (defun emacspeak-table-ui-filter-save (file)
   "Save out filter settings."
   (interactive
